==> js-language-23.1.1-sources.jar: com/oracle/truffle/js/nodes/binary/JSOrNode.java <==
/*
 * Copyright (c) 2018, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.binary;

import java.util.Set;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.Cached.Shared;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.instrumentation.Tag;
import com.oracle.truffle.api.nodes.NodeInfo;
import com.oracle.truffle.api.profiles.InlinedConditionProfile;
import com.oracle.truffle.js.nodes.JavaScriptNode;
import com.oracle.truffle.js.nodes.cast.JSToBooleanNode;
import com.oracle.truffle.js.runtime.objects.Undefined;

@NodeInfo(shortName = "||")
public abstract class JSOrNode extends JSLogicalNode {

    @Child private JSToBooleanNode toBooleanCast = JSToBooleanNode.create();

    JSOrNode(JavaScriptNode left, JavaScriptNode right) {
        super(left, right);
    }

    public static JavaScriptNode create(JavaScriptNode left, JavaScriptNode right) {
        return JSOrNodeGen.create(left, right);
    }

    public static JavaScriptNode createNotUndefinedOr(JavaScriptNode left, JavaScriptNode right) {
        return NotUndefinedOrNodeGen.create(left, right);
    }

    @Specialization
    protected Object doBoolean(VirtualFrame frame, boolean leftValue,
                    @Cached @Shared("profile") InlinedConditionProfile canShortCircuit) {
        if (canShortCircuit.profile(this, leftValue)) {
            return leftValue;
        } else {
            return rightNode.execute(frame);
        }
    }

    @Specialization(replaces = "doBoolean")
    protected Object doGeneric(VirtualFrame frame, Object leftValue,
                    @Cached(inline = true) JSToBooleanNode toBoolean,
                    @Cached @Shared("profile") InlinedConditionProfile canShortCircuit) {
        if (canShortCircuit.profile(this, toBoolean.executeBoolean(this, leftValue))) {
            return leftValue;
        } else {
            return rightNode.execute(frame);
        }
    }

    @Override
    protected JavaScriptNode copyUninitialized(Set<Class<? extends Tag>> materializedTags) {
        return create(cloneUninitialized(getLeft(), materializedTags), cloneUninitialized(getRight(), materializedTags));
    }
}

abstract class NotUndefinedOrNode extends JSLogicalNode {

    NotUndefinedOrNode(JavaScriptNode left, JavaScriptNode right) {
        super(left, right);
    }

    @Specialization
    protected Object doGeneric(VirtualFrame frame, Object leftValue,
                    @Cached InlinedConditionProfile canShortCircuit) {
        if (canShortCircuit.profile(this, leftValue != Undefined.instance)) {
            return leftValue;
        } else {
            return rightNode.execute(frame);
        }
    }

    @Override
    protected JavaScriptNode copyUninitialized(Set<Class<? extends Tag>> materializedTags) {
        return NotUndefinedOrNodeGen.create(cloneUninitialized(getLeft(), materializedTags), cloneUninitialized(getRight(), materializedTags));
    }
}

==> js-language-24.1.1-sources.jar: com/oracle/truffle/js/nodes/binary/JSOrNode.java <==
/*
 * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.binary;

import java.util.Set;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.Cached.Shared;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.instrumentation.Tag;
import com.oracle.truffle.api.nodes.NodeInfo;
import com.oracle.truffle.api.profiles.InlinedConditionProfile;
import com.oracle.truffle.js.nodes.JavaScriptNode;
import com.oracle.truffle.js.nodes.cast.JSToBooleanNode;
import com.oracle.truffle.js.runtime.objects.Undefined;

@NodeInfo(shortName = "||")
public abstract class JSOrNode extends JSLogicalNode {

    JSOrNode(JavaScriptNode left, JavaScriptNode right) {
        super(left, right);
    }

    public static JavaScriptNode create(JavaScriptNode left, JavaScriptNode right) {
        return JSOrNodeGen.create(left, right);
    }

    public static JavaScriptNode createNotUndefinedOr(JavaScriptNode left, JavaScriptNode right) {
        return NotUndefinedOrNodeGen.create(left, right);
    }

    @Specialization
    protected Object doBoolean(VirtualFrame frame, boolean leftValue,
                    @Cached @Shared InlinedConditionProfile canShortCircuit) {
        if (canShortCircuit.profile(this, leftValue)) {
            return leftValue;
        } else {
            return rightNode.execute(frame);
        }
    }

    @Specialization(replaces = "doBoolean")
    protected Object doGeneric(VirtualFrame frame, Object leftValue,
                    @Cached(inline = true) JSToBooleanNode toBoolean,
                    @Cached @Shared InlinedConditionProfile canShortCircuit) {
        if (canShortCircuit.profile(this, toBoolean.executeBoolean(this, leftValue))) {
            return leftValue;
        } else {
            return rightNode.execute(frame);
        }
    }

    @Override
    protected JavaScriptNode copyUninitialized(Set<Class<? extends Tag>> materializedTags) {
        return create(cloneUninitialized(getLeft(), materializedTags), cloneUninitialized(getRight(), materializedTags));
    }
}

abstract class NotUndefinedOrNode extends JSLogicalNode {

    NotUndefinedOrNode(JavaScriptNode left, JavaScriptNode right) {
        super(left, right);
    }

    @Specialization
    protected Object doGeneric(VirtualFrame frame, Object leftValue,
                    @Cached InlinedConditionProfile canShortCircuit) {
        if (canShortCircuit.profile(this, leftValue != Undefined.instance)) {
            return leftValue;
        } else {
            return rightNode.execute(frame);
        }
    }

    @Override
    protected JavaScriptNode copyUninitialized(Set<Class<? extends Tag>> materializedTags) {
        return NotUndefinedOrNodeGen.create(cloneUninitialized(getLeft(), materializedTags), cloneUninitialized(getRight(), materializedTags));
    }
}
